@@ -11,7 +11,6 @@
 #define S_ISREG(mode)  (((mode) & S_IFMT) == S_IFREG)
 #endif
 
-static char msg[4069];
 /*
 #ifdef _DEFINE_WIN32_FSEEKO
  int fseeko(FILE *stream, off_t offset, int whence) {
@@ -26,7 +25,7 @@ static char msg[4069];
 #endif
 */
 
-
+/*
 int saprintf(char **retp, const char *fmt, ...) {
   va_list argp;
   int len;
@@ -49,10 +48,12 @@ int saprintf(char **retp, const char *fmt, ...) {
   *retp = buf;
   return len;
 }
+*/
 
 //************************* Debug helpers **************************
 // Assert failed routine
 void GAssert(const char* expression, const char* filename, unsigned int lineno){
+  char msg[4096];
   sprintf(msg,"%s(%d): ASSERT(%s) failed.\n",filename,lineno,expression);
   fprintf(stderr,"%s",msg);
   //abort();
@@ -60,6 +61,7 @@ void GAssert(const char* expression, const char* filename, unsigned int lineno){
 // Error routine (prints error message and exits!)
 void GError(const char* format,...){
   #ifdef __WIN32__
+    char msg[4096];
     va_list arguments;
     va_start(arguments,format);
     vsprintf(msg,format,arguments);
@@ -82,6 +84,7 @@ void GError(const char* format,...){
   
 // Warning routine (just print message without exiting)
 void GMessage(const char* format,...){
+  char msg[4096];
   va_list arguments;
   va_start(arguments,format);
   vsprintf(msg,format,arguments);
@@ -163,23 +166,40 @@ char* Gstrdup(const char* sfrom, const char* sto) {
   return copy;
   }
 
-int Gstrcmp(char* a, char* b) {
+int Gstrcmp(const char* a, const char* b, int n) {
  if (a==NULL || b==NULL) {
    return a==NULL ? -1 : 1;
    }
- else return strcmp(a,b);
+ else {
+   if (n<0) return strcmp(a,b);
+       else return strncmp(a,b,n);
+ }
+
 }
 
-int Gstricmp(const char* a, const char* b) {
+int Gstricmp(const char* a, const char* b, int n) {
  if (a==NULL || b==NULL) return a==NULL ? -1 : 1;
  register int ua, ub;
- while ((*a!=0) && (*b!=0)) {
-  ua=tolower((unsigned char)*a);
-  ub=tolower((unsigned char)*b);
-  a++;b++;
-  if (ua!=ub) return ua < ub ? -1 : 1;
+ if (n<0) {
+   while ((*a!=0) && (*b!=0)) {
+    ua=tolower((unsigned char)*a);
+    ub=tolower((unsigned char)*b);
+    a++;b++;
+    if (ua!=ub) return ua < ub ? -1 : 1;
+    }
+    return (*a == 0) ? ( (*b == 0) ? 0 : -1 ) : 1 ;
+  }
+ else {
+   while (n && (*a!=0) && (*b!=0)) {
+    ua=tolower((unsigned char)*a);
+    ub=tolower((unsigned char)*b);
+    a++;b++;n--;
+    if (ua!=ub) return ua < ub ? -1 : 1;
+    }
+    //return (*a == 0) ? ( (*b == 0) ? 0 : -1 ) : 1 ;
+   if (n==0) return 0;
+   else { return (*a == 0) ? ( (*b == 0) ? 0 : -1 ) : 1 ; }
   }
-  return (*a == 0) ? ( (*b == 0) ? 0 : -1 ) : 1 ;
 }
 
 int strsplit(char* str, char** fields, int maxfields, const char* delim) {
@@ -266,7 +286,7 @@ char* Gsubstr(const char* str, char* from, char* to) {
     }
  if (to<from) return newEmptyStr();
  int newlen=to-from+1;
- char* subs;
+ char* subs=NULL;
  GMALLOC(subs, newlen);
  memcpy(subs, str, newlen-1);
  subs[newlen]='\0';
@@ -392,7 +412,7 @@ char* strchrs(const char* s, const char* chrs) {
 char* upCase(const char* str) {
  if (str==NULL) return NULL;
  int len=strlen(str);
- char* upstr;
+ char* upstr=NULL;
  GMALLOC(upstr, len+1);
  upstr[len]='\0';
  for (int i=0;i<len;i++) upstr[i]=toupper(str[i]);
@@ -402,7 +422,7 @@ char* upCase(const char* str) {
 char* loCase(const char* str) {
  if (str==NULL) return NULL;
  int len=strlen(str);
- char* lostr;
+ char* lostr=NULL;
  GMALLOC(lostr, len+1);
  lostr[len]='\0';
  for (int i=0;i<len;i++) lostr[i]=tolower(str[i]);
@@ -455,7 +475,7 @@ char* rstrfind(const char* str, const char* substr) {
 
 
 char* strifind(const char* str,  const char* substr) {
- // the case insensitive version of strstr -- finding a string within a strin
+ // case insensitive version of strstr -- finding a string within another
   int l,i;
   if (str==NULL || *str==0) return NULL;
   if (substr==NULL || *substr==0) return NULL;
@@ -734,8 +754,8 @@ void writeFasta(FILE *fw, const char* seqid, const char* descr,
  }
 
 char* commaprint(uint64 n) {
-  static int comma = '\0';
-  static char retbuf[48];
+  int comma = '\0';
+  char retbuf[48];
   char *p = &retbuf[sizeof(retbuf)-1];
   int i = 0;
   if(comma == '\0') {
