@@ -14,18 +14,15 @@
 #include <boost/numeric/ublas/io.hpp>
 
 #include <vector>
+#include <Eigen/Dense>
 
 namespace ublas = boost::numeric::ublas;
 
 double entropy(const ublas::vector<double>& p);
 
-double jensen_shannon_distance(std::vector<ublas::vector<double> >& sample_kappas);
+double jensen_shannon_distance(std::vector<Eigen::VectorXd>& sample_kappas);
 
-//void alt_jensen_shannon_gradient(std::vector<ublas::vector<double> >& sample_kappas,
-//                                 double js,
-//                                 ublas::vector<double>& gradient);
-
-void jensen_shannon_gradient(std::vector<ublas::vector<double> >& sample_kappas,
+void jensen_shannon_gradient(std::vector<Eigen::VectorXd>& sample_kappas,
 							 double js,
 							 ublas::vector<double>& gradient);
 
