@@ -407,7 +407,7 @@ void calc_quartile_scaling_factors(const vector<LocusCountList>& sample_compatib
         for (size_t j = 0; j < sample_compatible_count_table.size(); ++j)
         {
         
-            //shared_ptr<ReadGroupProperties> rg = bundle_factories[fac_idx];
+            //boost::shared_ptr<ReadGroupProperties> rg = bundle_factories[fac_idx];
             //double scaled_mass = scale_factors[fac_idx] * rg->total_map_mass();
             
             total_common += sample_compatible_count_table[j].counts[i];
@@ -619,7 +619,7 @@ void build_scv_correction_fit(int nreps, int ngenes, int mean_count, SCVInterpol
         vector<double> draws;
         for (size_t i = 0; i < ngenes; ++i)
         {
-            LocusCountList locus_count("", nreps, 1); 
+            LocusCountList locus_count("", nreps, 1, vector<string>(), vector<string>());
             for (size_t rep_idx = 0; rep_idx < nreps; ++rep_idx)
             {
                 double gamma_draw = gamma(rng);
@@ -764,8 +764,10 @@ fit_dispersion_model_helper(const string& condition_name,
     SCVInterpolator true_to_est_scv_table;
     
     int num_samples = sample_compatible_count_table.front().counts.size();
-    
-    build_scv_correction_fit(num_samples, 10000, 100000, true_to_est_scv_table);
+    if (no_scv_correction == false)
+    {
+        build_scv_correction_fit(num_samples, 10000, 100000, true_to_est_scv_table);
+    }
     
     setuplf();  
     
@@ -787,8 +789,8 @@ fit_dispersion_model_helper(const string& condition_name,
     
     if (compatible_count_means.size() < min_loci_for_fitting)
     {
-        shared_ptr<MassDispersionModel> disperser;
-        disperser = shared_ptr<MassDispersionModel>(new PoissonDispersionModel(condition_name));
+        boost::shared_ptr<MassDispersionModel> disperser;
+        disperser = boost::shared_ptr<MassDispersionModel>(new PoissonDispersionModel(condition_name));
         
         return disperser;
     }
@@ -865,10 +867,10 @@ fit_dispersion_model_helper(const string& condition_name,
 //        }
     }
     
-    shared_ptr<MassDispersionModel> disperser;
-    disperser = shared_ptr<MassDispersionModel>(new MassDispersionModel(condition_name, compatible_count_means, raw_variances, fitted_values));
+    boost::shared_ptr<MassDispersionModel> disperser;
+    disperser = boost::shared_ptr<MassDispersionModel>(new MassDispersionModel(condition_name, compatible_count_means, raw_variances, fitted_values));
     if (dispersion_method == POISSON)
-        disperser = shared_ptr<MassDispersionModel>(new PoissonDispersionModel(condition_name));
+        disperser = boost::shared_ptr<MassDispersionModel>(new PoissonDispersionModel(condition_name));
     
 //    for (map<string, pair<double, double> >::iterator itr = labeled_mv_table.begin();
 //         itr != labeled_mv_table.end();
@@ -895,7 +897,7 @@ fit_dispersion_model(const string& condition_name,
         if (sample_compatible_count_table[i].counts.size() <= 1)
         {
             // only one replicate - no point in fitting variance
-            return shared_ptr<MassDispersionModel>(new PoissonDispersionModel(condition_name));
+            return boost::shared_ptr<MassDispersionModel>(new PoissonDispersionModel(condition_name));
         }
     }
 #if ENABLE_THREADS
@@ -921,14 +923,68 @@ fit_dispersion_model(const string& condition_name,
     return model;
 }
 
-void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups)
+void build_norm_table(const vector<LocusCountList>& full_count_table,
+                      boost::shared_ptr<const map<string, LibNormStandards> > normalizing_standards,
+                      vector<LocusCountList>& norm_table)
+{
+    // If we're using housekeeping genes or spike-in controls, select the rows we'll be using from the full count table.
+    if (normalizing_standards)
+    {
+        for (size_t i = 0; i < full_count_table.size(); ++i)
+        {
+            const vector<string>& gene_ids = full_count_table[i].gene_ids;
+            const vector<string>& gene_short_names = full_count_table[i].gene_short_names;
+            
+            // If the row has an ID that's in the table, take it.
+            map<string, LibNormStandards>::const_iterator g_id_itr = normalizing_standards->end();
+            map<string, LibNormStandards>::const_iterator g_name_itr = normalizing_standards->end();
+            
+            for (size_t j = 0; j < gene_ids.size(); ++j)
+            {
+                g_id_itr = normalizing_standards->find(gene_ids[j]);
+                if (g_id_itr != normalizing_standards->end())
+                {
+                    break;
+                }
+            }
+            
+            if (g_id_itr != normalizing_standards->end())
+            {
+                norm_table.push_back(full_count_table[i]);
+                continue;
+            }
+
+            for (size_t j = 0; j < gene_short_names.size(); ++j)
+            {
+                g_name_itr = normalizing_standards->find(gene_short_names[j]);
+                if (g_name_itr != normalizing_standards->end())
+                {
+                    break;
+                }
+            }
+            
+            if (g_name_itr != normalizing_standards->end())
+            {
+                norm_table.push_back(full_count_table[i]);
+                continue;
+            }
+
+        }
+    }
+    else // otherwise, just take all rows.
+    {
+        norm_table = full_count_table;
+    }
+}
+
+void normalize_counts(vector<boost::shared_ptr<ReadGroupProperties> > & all_read_groups)
 {
     vector<LocusCountList> sample_compatible_count_table;
     vector<LocusCountList> sample_total_count_table;
     
     for (size_t i = 0; i < all_read_groups.size(); ++i)
     {
-        shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
+        boost::shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
         const vector<LocusCount>& raw_compatible_counts = rg_props->raw_compatible_counts();
         const vector<LocusCount>& raw_total_counts = rg_props->raw_total_counts();
         
@@ -938,8 +994,12 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
             {
                 const string& locus_id = raw_compatible_counts[j].locus_desc;
                 int num_transcripts = raw_compatible_counts[j].num_transcripts;
-                sample_compatible_count_table.push_back(LocusCountList(locus_id,all_read_groups.size(), num_transcripts));
-                sample_total_count_table.push_back(LocusCountList(locus_id,all_read_groups.size(), num_transcripts));
+                
+                const vector<string>& gene_ids = raw_compatible_counts[j].gene_ids;
+                const vector<string>& gene_short_names = raw_compatible_counts[j].gene_short_names;
+                
+                sample_compatible_count_table.push_back(LocusCountList(locus_id,all_read_groups.size(), num_transcripts, gene_ids, gene_short_names));
+                sample_total_count_table.push_back(LocusCountList(locus_id,all_read_groups.size(), num_transcripts, gene_ids, gene_short_names));
             }
             double scaled = raw_compatible_counts[j].count;
             //sample_compatible_count_table[j].counts[i] = scaled * unscaling_factor;
@@ -956,13 +1016,14 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
     
     if (use_compat_mass)
     {
-        norm_table = sample_compatible_count_table;
+        build_norm_table(sample_compatible_count_table, lib_norm_standards, norm_table);
     }
     else // use_total_mass
     {
         assert(use_total_mass);
-        norm_table = sample_total_count_table;
+        build_norm_table(sample_total_count_table, lib_norm_standards, norm_table);
     }
+    
     if (lib_norm_method == GEOMETRIC)
     {
         calc_geometric_scaling_factors(norm_table, scale_factors);
@@ -979,6 +1040,10 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
     {
         calc_tmm_scaling_factors(norm_table, scale_factors);
     }
+    else if (lib_norm_method == ESTIMATED_ABSOLUTE)
+    {
+        calc_classic_fpkm_scaling_factors(norm_table, scale_factors);
+    }
     else
     {
         assert (false);
@@ -988,7 +1053,7 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
     
     for (size_t i = 0; i < all_read_groups.size(); ++i)
     {
-        shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
+        boost::shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
         rg_props->internal_scale_factor(scale_factors[i]);
     }
     
@@ -1014,14 +1079,18 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
     
     for (size_t i = 0; i < all_read_groups.size(); ++i)
     {
-        shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
+        boost::shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
         vector<LocusCount> scaled_compatible_counts;
         for (size_t j = 0; j < sample_compatible_count_table.size(); ++j)
         {
             string& locus_id = sample_compatible_count_table[j].locus_desc;
             double count = sample_compatible_count_table[j].counts[i];
             int num_transcripts = sample_compatible_count_table[j].num_transcripts;
-            LocusCount locus_count(locus_id, count, num_transcripts);
+            
+            const vector<string>& gids = sample_compatible_count_table[j].gene_ids;
+            const vector<string>& gnms = sample_compatible_count_table[j].gene_short_names;
+            
+            LocusCount locus_count(locus_id, count, num_transcripts, gids, gnms);
             scaled_compatible_counts.push_back(locus_count);
         }
         rg_props->common_scale_compatible_counts(scaled_compatible_counts);
@@ -1029,14 +1098,18 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
     
     for (size_t i = 0; i < all_read_groups.size(); ++i)
     {
-        shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
+        boost::shared_ptr<ReadGroupProperties> rg_props = all_read_groups[i];
         vector<LocusCount> scaled_total_counts;
         for (size_t j = 0; j < sample_total_count_table.size(); ++j)
         {
             string& locus_id = sample_total_count_table[j].locus_desc;
             double count = sample_total_count_table[j].counts[i];
             int num_transcripts = sample_total_count_table[j].num_transcripts;
-            LocusCount locus_count(locus_id, count, num_transcripts);
+            
+            const vector<string>& gids = sample_total_count_table[j].gene_ids;
+            const vector<string>& gnms = sample_total_count_table[j].gene_short_names;
+            
+            LocusCount locus_count(locus_id, count, num_transcripts, gids, gnms);
             scaled_total_counts.push_back(locus_count);
         }
         rg_props->common_scale_total_counts(scaled_total_counts);
@@ -1066,7 +1139,7 @@ void normalize_counts(vector<shared_ptr<ReadGroupProperties> > & all_read_groups
         avg_total_common_scaled_count += (1.0/all_read_groups.size()) * total_common;
     }
     
-    BOOST_FOREACH(shared_ptr<ReadGroupProperties> rg, all_read_groups)
+    BOOST_FOREACH(boost::shared_ptr<ReadGroupProperties> rg, all_read_groups)
     {
         rg->normalized_map_mass(avg_total_common_scaled_count);
     }
