@@ -116,11 +116,13 @@ private:
 
 struct LocusCountList
 {
-    LocusCountList(std::string ld, int num_reps, int nt) : 
-    locus_desc(ld), counts(std::vector<double>(num_reps, 0)), num_transcripts(nt) {}
+    LocusCountList(std::string ld, int num_reps, int nt, const std::vector<std::string>& gids, const std::vector<std::string>& gnms) :
+    locus_desc(ld), counts(std::vector<double>(num_reps, 0)), num_transcripts(nt), gene_ids(gids), gene_short_names(gnms) {}
     std::string locus_desc;
     std::vector<double> counts;
     int num_transcripts;
+    vector<std::string> gene_ids;
+    vector<std::string> gene_short_names;
 };
 
 void transform_counts_to_common_scale(const vector<double>& scale_factors,
@@ -151,7 +153,7 @@ void calculate_count_means_and_vars(const vector<LocusCountList>& sample_compati
 class ReplicatedBundleFactory
 {
 public:
-	ReplicatedBundleFactory(const std::vector<shared_ptr<BundleFactory> >& factories, 
+	ReplicatedBundleFactory(const std::vector<boost::shared_ptr<BundleFactory> >& factories, 
                             const string& condition_name)
     : _factories(factories), _condition_name(condition_name) {}
 	
@@ -191,21 +193,35 @@ public:
             }
         }
         
-        if (non_empty_bundle == false)
+        int locus_id = -1;
+        for (size_t i = 0; i < bundles.size(); ++i)
         {
-            BOOST_FOREACH (HitBundle* in_bundle, bundles)
+            if (locus_id == -1)
+                locus_id = bundles[i]->id();
+            if (locus_id != bundles[i]->id())
             {
-                in_bundle->ref_scaffolds().clear();
-                in_bundle->clear_hits();
-                delete in_bundle;
+                fprintf(stderr, "Error: locus id mismatch!\n");
+                exit(1);
             }
-            return false;
         }
         
+        
+//        if (non_empty_bundle == false)
+//        {
+//            bundle_out.id(locus_id);
+//            BOOST_FOREACH (HitBundle* in_bundle, bundles)
+//            {
+//                in_bundle->ref_scaffolds().clear();
+//                in_bundle->clear_hits();
+//                delete in_bundle;
+//            }
+//            return false;
+//        }
+        
         for (size_t i = 1; i < bundles.size(); ++i)
         {
-            const vector<shared_ptr<Scaffold> >& s1 = bundles[i]->ref_scaffolds();
-            const vector<shared_ptr<Scaffold> >& s2 =  bundles[i-1]->ref_scaffolds();
+            const vector<boost::shared_ptr<Scaffold> >& s1 = bundles[i]->ref_scaffolds();
+            const vector<boost::shared_ptr<Scaffold> >& s2 =  bundles[i-1]->ref_scaffolds();
             assert (s1.size() == s2.size());
             for (size_t j = 0; j < s1.size(); ++j)
             {
@@ -213,16 +229,30 @@ public:
             }
         }
         
+        double total_compatible_mass = 0.0;
+        double total_raw_mass = 0.0;
+        
+        for (size_t i = 0; i < bundles.size(); ++i)
+        {
+            total_compatible_mass += bundles[i]->compatible_mass();
+            total_raw_mass += bundles[i]->raw_mass();
+        }
+        
         // Merge the replicates into a combined bundle of hits.
         HitBundle::combine(bundles, bundle_out);
         
+        bundle_out.compatible_mass(total_compatible_mass);
+        bundle_out.add_raw_mass(total_raw_mass);
+        
+        bundle_out.id(locus_id);
+        
         BOOST_FOREACH (HitBundle* in_bundle, bundles)
         {
             in_bundle->ref_scaffolds().clear();
             in_bundle->clear_hits();
             delete in_bundle;
         }
-        return true;
+        return non_empty_bundle;
     }
 	
 	void reset() 
@@ -230,7 +260,7 @@ public:
 #if ENABLE_THREADS
         boost::mutex::scoped_lock lock(_rep_factory_lock);
 #endif
-        BOOST_FOREACH (shared_ptr<BundleFactory> fac, _factories)
+        BOOST_FOREACH (boost::shared_ptr<BundleFactory> fac, _factories)
         {
             fac->reset();
         }
@@ -245,14 +275,14 @@ public:
         
         for (size_t fac_idx = 0; fac_idx < _factories.size(); ++fac_idx)
         {
-            shared_ptr<BundleFactory> fac = _factories[fac_idx];        
+            boost::shared_ptr<BundleFactory> fac = _factories[fac_idx];        
             BadIntronTable bad_introns;
             
             vector<LocusCount> compatible_count_table;
             vector<LocusCount> total_count_table;
-            inspect_map(*fac, NULL, compatible_count_table, total_count_table, false, false);
+            inspect_map(fac, NULL, compatible_count_table, total_count_table, false, false);
             
-            shared_ptr<ReadGroupProperties> rg_props = fac->read_group_properties();
+            boost::shared_ptr<ReadGroupProperties> rg_props = fac->read_group_properties();
             
             assert (compatible_count_table.size() == total_count_table.size());
             
@@ -263,7 +293,7 @@ public:
                 
                 if (i >= sample_compatible_count_table.size())
                 {
-                    LocusCountList locus_count(c.locus_desc, _factories.size(), c.num_transcripts); 
+                    LocusCountList locus_count(c.locus_desc, _factories.size(), c.num_transcripts, c.gene_ids, c.gene_short_names);
                     sample_compatible_count_table.push_back(locus_count);
                     sample_compatible_count_table.back().counts[0] = raw_count;
                     sample_total_count_table.push_back(locus_count);
@@ -296,7 +326,7 @@ public:
         
         for (size_t i = 0; i < scale_factors.size(); ++i)
         {
-            shared_ptr<ReadGroupProperties> rg_props = _factories[i]->read_group_properties();
+            boost::shared_ptr<ReadGroupProperties> rg_props = _factories[i]->read_group_properties();
             assert (scale_factors[i] != 0);
             rg_props->internal_scale_factor(scale_factors[i]);
         }
@@ -306,7 +336,7 @@ public:
         
         for (size_t fac_idx = 0; fac_idx < _factories.size(); ++fac_idx)
         {
-            shared_ptr<ReadGroupProperties> rg_props = _factories[fac_idx]->read_group_properties();
+            boost::shared_ptr<ReadGroupProperties> rg_props = _factories[fac_idx]->read_group_properties();
             assert (scale_factors[fac_idx] != 0);
             vector<LocusCount> common_scaled_compatible_counts;
             vector<LocusCount> common_scaled_total_counts;
@@ -335,9 +365,9 @@ public:
         
         for (size_t fac_idx = 0; fac_idx < _factories.size(); ++fac_idx)
         {
-            shared_ptr<BundleFactory> fac = _factories[fac_idx];        
+            boost::shared_ptr<BundleFactory> fac = _factories[fac_idx];        
             
-            shared_ptr<ReadGroupProperties> rg_props = fac->read_group_properties();
+            boost::shared_ptr<ReadGroupProperties> rg_props = fac->read_group_properties();
             const vector<LocusCount>& compatible_count_table = rg_props->common_scale_compatible_counts();
             const vector<LocusCount>& total_count_table = rg_props->common_scale_total_counts();
             
@@ -351,7 +381,7 @@ public:
                 
                 if (i >= sample_compatible_count_table.size())
                 {
-                    LocusCountList locus_count(c.locus_desc, _factories.size(), c.num_transcripts); 
+                    LocusCountList locus_count(c.locus_desc, _factories.size(), c.num_transcripts, c.gene_ids, c.gene_short_names);
                     sample_compatible_count_table.push_back(locus_count);
                     sample_compatible_count_table.back().counts[0] = common_scale_compatible_count;
                     sample_total_count_table.push_back(locus_count);
@@ -371,7 +401,7 @@ public:
             scale_factors.push_back(rg_props->internal_scale_factor());
         }
         
-        shared_ptr<MassDispersionModel> disperser;
+        boost::shared_ptr<MassDispersionModel> disperser;
         disperser = ::fit_dispersion_model(_condition_name, scale_factors, sample_compatible_count_table);
         
         vector<pair<double, double> > compatible_means_and_vars;
@@ -401,9 +431,9 @@ public:
         }
 
         
-        BOOST_FOREACH (shared_ptr<BundleFactory> fac, _factories)
+        BOOST_FOREACH (boost::shared_ptr<BundleFactory> fac, _factories)
         {
-            shared_ptr<ReadGroupProperties> rg_props = fac->read_group_properties();
+            boost::shared_ptr<ReadGroupProperties> rg_props = fac->read_group_properties();
             rg_props->mass_dispersion_model(disperser);
         }
     }
@@ -411,23 +441,23 @@ public:
     
     // This function NEEDS to deep copy the ref_mRNAs, otherwise cuffdiff'd
     // samples will clobber each other
-    void set_ref_rnas(const vector<shared_ptr<Scaffold> >& mRNAs)
+    void set_ref_rnas(const vector<boost::shared_ptr<Scaffold> >& mRNAs)
     {
 #if ENABLE_THREADS
         boost::mutex::scoped_lock lock(_rep_factory_lock);
 #endif
-        BOOST_FOREACH(shared_ptr<BundleFactory> fac, _factories)
+        BOOST_FOREACH(boost::shared_ptr<BundleFactory> fac, _factories)
         {
             fac->set_ref_rnas(mRNAs);
         }
     }
     
-    void set_mask_rnas(const vector<shared_ptr<Scaffold> >& mRNAs)
+    void set_mask_rnas(const vector<boost::shared_ptr<Scaffold> >& mRNAs)
     {
 #if ENABLE_THREADS
         boost::mutex::scoped_lock lock(_rep_factory_lock);
 #endif
-        BOOST_FOREACH(shared_ptr<BundleFactory> fac, _factories)
+        BOOST_FOREACH(boost::shared_ptr<BundleFactory> fac, _factories)
         {
             fac->set_mask_rnas(mRNAs);
         }
@@ -435,40 +465,40 @@ public:
     
     int num_replicates() const { return _factories.size(); }
     
-    void mass_dispersion_model(shared_ptr<MassDispersionModel const> disperser)
+    void mass_dispersion_model(boost::shared_ptr<MassDispersionModel const> disperser)
     {
 #if ENABLE_THREADS
         boost::mutex::scoped_lock lock(_rep_factory_lock);
 #endif
-        BOOST_FOREACH(shared_ptr<BundleFactory>& fac, _factories)
+        BOOST_FOREACH(boost::shared_ptr<BundleFactory>& fac, _factories)
         {
             fac->read_group_properties()->mass_dispersion_model(disperser);
         }
     }
     
-    shared_ptr<MassDispersionModel const> mass_dispersion_model() const
+    boost::shared_ptr<MassDispersionModel const> mass_dispersion_model() const
     {
         return _factories.front()->read_group_properties()->mass_dispersion_model();
     }
     
-    void mle_error_model(shared_ptr<MleErrorModel const> mle_model)
+    void mle_error_model(boost::shared_ptr<MleErrorModel const> mle_model)
     {
 #if ENABLE_THREADS
         boost::mutex::scoped_lock lock(_rep_factory_lock);
 #endif
-        BOOST_FOREACH(shared_ptr<BundleFactory>& fac, _factories)
+        BOOST_FOREACH(boost::shared_ptr<BundleFactory>& fac, _factories)
         {
             fac->read_group_properties()->mle_error_model(mle_model);
         }
     }
     
-    shared_ptr<MleErrorModel const> mle_error_model() const
+    boost::shared_ptr<MleErrorModel const> mle_error_model() const
     {
         return _factories.front()->read_group_properties()->mle_error_model();
     }
     
 private:
-	vector<shared_ptr<BundleFactory> > _factories;
+	vector<boost::shared_ptr<BundleFactory> > _factories;
 #if ENABLE_THREADS
     boost::mutex _rep_factory_lock;
 #endif
